////Required Devices
//Cable Line Connector Assorted Kit
//https://www.amazon.com/gp/product/B09FP6T7CN
//LCD Screen Module for Arduino
//https://www.amazon.com/gp/product/B0BWTFN9WF
//A3144 Hall Effect Sensor 3Pins
//https://www.amazon.com/gp/product/B09PG3PGH6
//Thermistor Module
//https://www.amazon.com/gp/product/B09Z2J12GV
//Load Cell
//https://www.amazon.com/gp/product/B096FKK8KS
//Motor Controller
//https://www.amazon.com/gp/product/B08QJMRDYT
//10pcs Ball Bearings
//https://www.amazon.com/gp/product/B0CCNDW9JR
//Carbon Fiber Rod  1/4"
//https://www.fixsupply.com/bulk-cr-cf-4-carbon-fiber-rod-unidirectional-1-4-diameter-x-2-ft-long

/////////////////////////////////
// IIC LCD Display YT TUTORIAL //
//       ArduinoCraft          //
/////////////////////////////////
// library connection
#include <Wire.h>
#include <HX711.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 16, 2);   // Set the LCD address to 0x27 for a 16 chars and 2 line display


//Pins...................................................
int hall_pin = 4; // Hall sensor at pin 4
const int LOADCELL_DOUT_PIN = 3;
const int LOADCELL_SCK_PIN = 2;

//Timestamps.............................................
long current_time_stamp = 0;
long old_time_stamp = 0;

//Hall Sensor............................................
unsigned long rpm = 0;
bool on_state = false;
int hall_count = 0;


//Load Cell..............................................
HX711 scale;
float grams_to_lbs = 0.00220462;
float lbs_to_grams = 453.592;
float calibrate_weight_lbs = 10;
float scale_lbs = 0;
float scale_reading_grams = 0;
float scale_offset = -9214.69;
bool calibrate_scale_on = false;
float arm_m = 0.075;
float torque_nm = 0;
float radius_kg = 0;
float kilograms = 0;


//Thermal.................................................
int temp_pin = A0; // Analog PIN A0
int fahrenheit = 0;


//Setup........................................................
void setup() {
  Serial.begin(115200);
  lcd.init();       // Initialize the lcd
  lcd.backlight();  // turn on the backlight
  pinMode(hall_pin, INPUT); //hall Sensor input
  pinMode (temp_pin, INPUT);
  scale.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);


  //Recalibrate scale?......
  if (calibrate_scale_on) {
    calibrate_scale();
  } else {
    set_scale_offset();
  }


}


//Loop........................................................
void loop() {

  count_hall_sensor();
  get_scale_units();
  heartbeat();

}

//get temperature............................................
void get_temp() {

  int reading = analogRead(temp_pin);
  reading = map(reading, 1023, -55, 0, 125);
  fahrenheit = (reading * 9 / 5) - 32;
  Serial.print(fahrenheit);
  Serial.println(" degrees F");

}


void set_scale_offset()
{
  scale.set_scale();
  scale.tare();
  scale.set_scale(scale_offset);

  Serial.print(scale_offset);
  Serial.println(" offset");
}

void calibrate_scale() {

  if (scale.is_ready()) {
    scale.set_scale();
    Serial.println("Remove any weights from the scale.");
    delay(5000);
    scale.tare();
    Serial.println("Done...");
    Serial.print("Place a known weight on the scale...");
    delay(5000);
    long reading = scale.get_units(10) * 100;
    Serial.print("Result: ");
    Serial.println(reading);
    float calibration_factor = reading / (lbs_to_grams * calibrate_weight_lbs);
    Serial.println(calibration_factor);

    set_scale_offset();

  }
  else {
    Serial.println("HX711 not found.");
  }
  delay(1000);

}


void get_scale_units() {
  if (scale.is_ready()) {
    float grams = scale.get_units() * 100;

    //scale gear bounces so only record highest number.
    if (scale_reading_grams < grams)
    {
      scale_reading_grams = grams;
    }
  }


}

void calculate_torque() {



  scale_lbs = scale_reading_grams * grams_to_lbs;

  Serial.print(scale_lbs);
  Serial.println(" lbs");

  Serial.print(scale_reading_grams);
  Serial.println(" grams");

  //https://www.calculatorsoup.com/calculators/physics/force.php
  //Force =  mass * acceleration
  //Force = grams / 9.81 m/s squared * sin(0)

  //Torque = Force * radius * sin(θ)
  //Torque = Force * radius


  //mass in grams,  multiply by 9.81, then multiply by millimeter arm length
  //90 degree radians = 1.5708;

  //Kilograms = grams/1000
  //9.81 squared = 96.2361

  //To convert grams to newtons, divide the mass in grams by 1,000 and then multiply by 9.81.
  //For example, 100 grams is equal to 0.1 kilograms, and 0.1 kilograms multiplied by 9.8 meters per second squared is 0.98 newtons.
  kilograms = scale_reading_grams / 1000;
  //feet to meters
  torque_nm =  kilograms * 9.81 * arm_m;


  Serial.print(torque_nm);
  Serial.println(" torque_nm");

  //reset scale reading...............
  scale_reading_grams = 0;




}


//Send heartbeat every second.................................
void heartbeat() {
  current_time_stamp = millis();

  if (current_time_stamp  > old_time_stamp + 1000) {
    calculate_torque();
    get_temp();
    update_lcd_screen();
    //auto_calibrate_scale();
    old_time_stamp = current_time_stamp;

  }
}

//Hall Sensor revolutions counter.................................
void count_hall_sensor() {
  //how many times did hall reading change between heart beats?
  if (digitalRead(hall_pin) == 0) {
    if (on_state == false) {
      on_state = true;
      hall_count += 1.0;
     
    }
  } else {
    on_state = false;
  }

}


//auto calibrate scale....................................
void auto_calibrate_scale() {

  if (hall_count == 0 && !calibrate_scale_on) {

    if (scale_lbs > 0.1) {
      scale_offset = scale_offset + 100;
      set_scale_offset();
    } else if (scale_lbs < -0.1) {
      scale_offset = scale_offset - 100;
      set_scale_offset();
    }

  }


}

//Update LCD screen display.................................
void update_lcd_screen() {

  rpm = hall_count * 60; //See above
  lcd.clear();
  lcd.setCursor(0, 0);        // Set cursor to column 2 and line 1
  lcd.print("RPM:");
  lcd.print(rpm);
  lcd.print(" ");
  lcd.print(fahrenheit);
  lcd.print("F");


  lcd.setCursor(0, 1);        // Set cursor to column 2 and line 1
  lcd.print("Nm:");
  lcd.print(torque_nm, 4);
  hall_count = 0;
}
